//-------------------------------------------//
//  LWeapTweak by MrLoathsome.    6/21/2018  //
//-------------------------------------------//

class LWT extends Mutator config(LWeapTweak);

var() config bool bDebug;
var() config float DamageMult, SPDamMult, ChunkDamage;
var() config int MaxAdjVal;

struct DefDamType
{
	var string WeapC;
	var float PDamage;
	var float SDamage;
	var int HDamage;
};

var() config DefDamType WCList[255];

var int WeapCount;
var bool bInConfig;

function PostBeginPlay()
{

	CountWeaps();   // Get current count of weapons in ini.   List if bDebug
	Class'LWeapTweak.MyWatcher'.default.ChunkDamage = ChunkDamage;
	if (!bInConfig)
	{
		Spawn(class'MyWatcher',Self);  // Don't spawn the MyWatcher class if called by Mod Menu
		Level.Game.RegisterDamageMutator(Self);  // Don't register if called by Mod Menu
	}
	Super.PostBeginPlay();
}

function bool CheckWeap(Weapon W)  //  Called by the MyWatcher class.  If weap exists in ini adjusts weapon and returns false.   Else adds weap to ini file with class default damage values.
{
	local int i;

	for (i=0; WCList[i].WeapC != ""; i++)
	{
		if (class<Weapon>(DynamicLoadObject(WCList[i].WeapC, class'Class')) == W.Class)
		{
			SetWeap(W, i);  // Weapon exists...  Adjust the Weapon
			return false;
		}
	}
	if (i < 255)
	{
		WCList[i].WeapC = String(W.Class);  //  New Weapon...
		if (W.ProjectileClass != None) { WCList[i].PDamage = W.ProjectileClass.default.damage; }
		if (W.AltProjectileClass != None) { WCList[i].SDamage = W.AltProjectileClass.default.damage; }
		if (W.GetPropertyText("hitdamage") != "") WCList[i].HDamage = int(W.GetPropertyText("hitdamage"));
		if (bDebug) log("LWT - New Weapon Class Detected:"@W.Class);
		SaveConfig();  // Save the new weapon in ini file with its default damage values
	}
	else
	{
		Warn("LWT - List Full.  Not Adding "@String(W.Class));
	}
	return true;
}


function CountWeaps()  //  called at startup.   Counts list of weapons classes from ini and lists if bDebug=True
{
	local int j;

	for (j=0; WCList[j].WeapC != ""; j++)
	{
		if ((bDebug) && (!bInConfig)) log("LWT ResetWeaps -  Default Damage:"@WCList[j].WeapC@WCList[j].PDamage@WCList[j].SDamage@WCList[j].HDamage);
	}
	WeapCount = j;
}

function SetWeap(out Weapon W, int i)  // Set the ini settings on the cool weapon....
{
	if (W.ProjectileClass != None) W.ProjectileClass.default.damage = WCList[i].PDamage;
	if (W.AltProjectileClass != None) W.AltProjectileClass.default.damage = WCList[i].SDamage;
	if (W.GetPropertyText("hitdamage") != "") W.SetPropertyText("hitdamage", string(WCList[i].HDamage));
}

function MutatorTakeDamage( out int ActualDamage, Pawn Victim, Pawn InstigatedBy, out Vector HitLocation, 
						out Vector Momentum, name DamageType)
{
	if ((InstigatedBy != None) && (ActualDamage > 1))
	{
		if ((ScriptedPawn(InstigatedBy) == None) && (ActualDamage < MaxAdjVal))  // adj low damage weaps for players/bots
		{
			ActualDamage *= DamageMult;
		}
		if ((ScriptedPawn(InstigatedBy) != None) && (ScriptedPawn(Victim) == None)) // adj for Monsters damage vs player/bots
		{
			ActualDamage *= SPDamMult;
		}
		if (bDebug)
		{
			if (((ScriptedPawn(InstigatedBy) == None) && (ScriptedPawn(Victim) == None)) || ((ScriptedPawn(InstigatedBy) == None) && (ScriptedPawn(Victim) != None)) || ((ScriptedPawn(InstigatedBy) != None) && (ScriptedPawn(Victim) == None)))
				log("LWT MutatorTakeDamage - Victim:"@Victim.Name$chr(9)$"InstigatedBy:"@InstigatedBy.Name$chr(9)$"DamageType:"@DamageType$chr(9)$"ActualDamage:"@ActualDamage);
		}
	}
	Super.MutatorTakeDamage( ActualDamage, Victim, InstigatedBy, HitLocation, Momentum, DamageType );
}


// some functions for the Mod Menu class to call....

function DefDamType GetWCListElement(int i)   //  Used by Menu to fetch values from specific WCList array element
{
	if (WCList[i].WeapC != "") return WCList[i];
}

function SetWCListWeapC(int i, string WC)  // Used by Menu to set WeapC for specific WCList array element
{
	WCList[i].WeapC = WC;
}

function SetWCListPD(int i, float PD)  // Used by Menu to set PDamage for specific WCList array element
{
	WCList[i].PDamage = PD;
}

function SetWCListSD(int i, float SD)  // Used by Menu to set SDamage for specific WCList array element
{
	WCList[i].SDamage = SD;
}

function SetWCListHD(int i, int HD)  // Used by Menu to set HDamage for specific WCList array element
{
	WCList[i].HDamage = HD;
}

defaultproperties
{
     bDebug=False
     DamageMult=1.0
     SPDamMult=1.0
     MaxAdjVal=25
     ChunkDamage=16
     bInConfig=False
}
